/*
 * dataview.cpp 
 *
 *  Copyright (C) 1999 Laszlo Boloni
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <kconfigbase.h>
#include <kconfig.h>
#include "dataview.h"
#include "dataview.moc"
#include <qlayout.h>
#include <klocale.h>


KDataviewConfig::KDataviewConfig( QWidget *parent, const char* name )
    : KCModule(parent, name) {
    layout = new QVBoxLayout(this);
    QGridLayout *glay = new QGridLayout(layout, 0, 0, 10); 
    QLabel *label;
    int row=0;

    maxItemsInView = new QSpinBox(5,50,1,this);
    connect(maxItemsInView, SIGNAL(valueChanged(int)),
	    this, SLOT(slotChanged()));
    glay->addWidget(maxItemsInView, row, 2);
    label = new QLabel(maxItemsInView, i18n("Maximum items in view"), this);
    glay->addWidget(label, row, 1);    
    row++;

    autosave = new QCheckBox(this);
    connect(autosave, SIGNAL(toggled(bool)),
	    this, SLOT(slotChanged()));
    glay->addWidget(autosave, row, 2);
    label = new QLabel(autosave, i18n("Automatic save?"), this);
    glay->addWidget(label, row, 1);
    row++;
    
    autosaveInterval = new QSpinBox(1,60,1,this);
    connect(autosaveInterval, SIGNAL(valueChanged(int)),
	    this, SLOT(slotChanged()));
    glay->addWidget(autosaveInterval, row, 2);
    label = new QLabel(autosaveInterval, i18n("Auto save interval (min)"), this);
    glay->addWidget(label, row, 1);
    row++;

    glay->addColSpacing(0,15);
    glay->setColStretch(0,0);
    glay->setColStretch(1,5);
    glay->setColStretch(2,1);
    glay->setColStretch(3,3);

    layout->activate();
}

KDataviewConfig::~KDataviewConfig( ) {
}

void KDataviewConfig::load() {
  config->setGroup("UserInterface");
  maxItemsInView->setValue( config->readNumEntry("maxItemsInView", 10));
  config->setGroup("Files");
  autosave->setChecked(config->readBoolEntry("autosave"));
  int temp = config->readNumEntry("autosaveInterval", 10);
  autosaveInterval->setValue(temp / 60000);
}

void KDataviewConfig::applySettings() {
    save();
}

void KDataviewConfig::save() {
    config->setGroup("UserInterface");
    config->writeEntry("maxItemsInView",maxItemsInView->value());
    config->setGroup("Files");
    config->writeEntry("autosave", autosave->isChecked());
    config->writeEntry("autosaveInterval",autosaveInterval->value()*60000);
}

void KDataviewConfig::defaults() {
}

void KDataviewConfig::slotChanged() {
  emit changed(true);
}
