/*
    This file is part of the KDE system
    Copyright (C)  1999 Boloni Laszlo

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.


    gofaiDataQuery.cpp

 */

#include "gofaiDataQuery.h"
#include <kdebug.h>

/**
   Constructor: attach the query object to a view
 */
gofaiDataQuery::gofaiDataQuery(gofaiDataView *dv) {
    this->dv = dv;
}

void gofaiDataQuery::addFromNeighbors(gofaiItem *it,
                      int level) {

    QList<gofaiItem> links;
    dv->ds->getAdjacentLinks(it, gofaiDataSource::From, links);
    QListIterator<gofaiItem> iter(links);
    for ( ; (it=iter.current()) ; ++iter ) {
        if (dv->addToView(dv->ds->lookupItem(it->to), level)) {
            dv->addToView(it, level);
        } else {
            // cerr << "Couln't add the node, for some reason!\n";
        }
    }
};

void gofaiDataQuery::addToNeighbors(gofaiItem *it,
                    int level) {
    QList<gofaiItem> links;
    dv->ds->getAdjacentLinks(it, gofaiDataSource::To, links);
    QListIterator<gofaiItem> iter(links);
    for ( ; (it=iter.current()); ++iter ) {
        if (dv->addToView(dv->ds->lookupItem(it->from), level)) {
            dv->addToView(it, level);
        } else {
            // cerr << "Couln't add the node, for some reason!\n";
        }
    }
};

void gofaiDataQuery::addNeighbors(gofaiItem *it,
                    int level) {
    addFromNeighbors(it, level);
    addToNeighbors(it, level);
}

void gofaiDataQuery::addInterlinks() {
  // FIXME: it doesn't add links between the context
  // FIXME: it adds all the links??? even those not in the view
    QList<gofaiItem> links;
    gofaiItem *it;
    for(int i=0; i!= dv->count; i++) {
      if (dv->vds[i]->val->type == Node) {
        dv->ds->getAdjacentLinks(dv->vds[i]->val,
                                 gofaiDataSource::Both,
                                 links);
      }
    }
    QListIterator<gofaiItem> iter(links);
    for ( ; (it=iter.current()); ++iter ) {
        if (dv->containsItem(it->from, true) &&
            dv->containsItem(it->to, true)) {
            bool f = dv->addToView(it, 1);
            if (!f) {
                kdDebug() << "Cannot add this link for some reason" << endl;
            }
        }
    }
};


void gofaiDataQuery::getView() {
    gofaiItem *it;
    dv->saveOldToTemp();
    // if no center, the first one
 again:
    if (dv->center == NULL) {
      kdDebug() << "gofaiDataQuery: No center, create one" << endl;
      it = dv->getItemById("");
      dv->addToView(it, 0);
      dv->center = dv->vds[0];
    } else {
        // first thing: add the center as level 0
        dv->addToView(dv->center->val, 0);
    }
    addNeighbors(dv->center->val, 1);
    // now add the neighbors of the context stuff
    for(int i=0; i!=dv->context_count; i++) {
        if (dv->context_vds[i]->val->type == Node) {
            addNeighbors(dv->context_vds[i]->val, 1);
        }
    }
    // repeat for first level
    for(int i=0; i!=dv->count; i++) {
        if (dv->vds[i]->val->type == Node) {
            addNeighbors(dv->vds[i]->val, 2);
        }
    }
    // repeat for second level
    for(int i=0; i!=dv->count; i++) {
        if (dv->vds[i]->val->type == Node) {
            addNeighbors(dv->vds[i]->val, 3);
        }
    }
    // and now add all the links between things in the stuff
    addInterlinks();
    dv->removeTemp();
    // test if we have at least one in the center:
    if (dv->count == 0) {
        kdDebug() << "No element remained!\n";
        dv->center = NULL;
        goto again;
    }
};
