/*
    This file is part of the KDE system
    Copyright (C)  1999 Boloni Laszlo

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

    gofaiDataView.h

    This allows a limited view of the whole data structure

*/

#ifndef __GOFAI_DATAVIEW_H
#define __GOFAI_DATAVIEW_H

#include "gofaiItem.h"
#include "gofaiVisualDataStruct.h"
#include "gofaiDataSource.h"
#include "gofaiDataViewPointer.h"
#include <qlist.h>

#define MAX_ITEMS_IN_VIEW 100
#define MAX_LEVELS 5
#define MAX_CONTEXT_ITEMS_IN_VIEW 100

/**
   This class represents the currently visible part of the
   data source.
 */

class gofaiConcentricArrangement;
class gofaiVisualArrangement;
class gofaiPosition;
class gofaiDataQuery;

class gofaiDataView :
public Domifiable {
  friend gofaiVisualArrangement;
  friend gofaiConcentricArrangement;
  friend gofaiDataQuery;
  friend gofaiPosition;
  friend gofaiDataViewPointer;
 public:
    // constructor destructor
   gofaiDataView();
   virtual ~gofaiDataView() {};
    // this will be made private, at some moment
   gofaiVisualItem *center;
    //

   void getView();
   int createVisuals(int centerx, int centery, int r);

    // domifiable
   bool writeToDOM(QDomDocument &doc, QDomElement &parent);
   bool readFromDOM(QDomElement &e);

    // data manipulation - bridge between the source
    void newGoph();
    virtual bool isChanged();
    virtual void addItem(gofaiItem *it);
    virtual void removeItem(gofaiItem *it);
    virtual void updateItem(gofaiItem *it);
    gofaiItem* getItemByName(QString name); // to be replaced?
    gofaiItem* getItemById(gofaiID id);
    void getMatchingItems(QString prefix, QList<gofaiItem> &list);


    // visual data
    gofaiVisualItem* containsItem(gofaiItem *it, bool context = false);
    gofaiVisualItem* containsItem(gofaiID id, bool context = false);
    void getNodeFromHere(const QPoint &pos, gofaiDataViewPointer &result);
    void getLinkFromHere(const QPoint &pos, gofaiDataViewPointer &result);
    void setCenter(gofaiVisualItem *vit);
    void setActive(gofaiItem *it);
    bool addToView(gofaiItem *it, int level);
    bool addToContext(gofaiVisualItem *vit);
    bool removeFromContext(gofaiDataViewPointer &dvp);
    bool drawIt(QPainter *p);

    QList <gofaiItem> history;

 private:
   // used for saving during query
   int localviews;
   int count;
   gofaiDataSource *ds;

   gofaiVisualItem *vds[MAX_ITEMS_IN_VIEW];
   int context_count;
   gofaiVisualItem *context_vds[MAX_CONTEXT_ITEMS_IN_VIEW];
   gofaiVisualItem *lvds[MAX_ITEMS_IN_VIEW];
   gofaiDataQuery *dq;

   void saveOldToTemp();
   void removeTemp();
   int countConnections(gofaiVisualItem *vit);
   void setNewPosition();

   void getItemFromHere(const QPoint &pos,
                        gofaiDataViewPointer &result,
                        int type);

};

#endif // __GOFAI_DATAVIEW_H
