/*
    This file is part of the KDE system
    Copyright (C)  1999, 2000 Boloni Laszlo

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

    gofaiWidget.cc
*/

#include <fstream.h>
#include <qevent.h>
#include <qpainter.h>
#include <qdragobject.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kdebug.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>
#include <kiconloader.h>
#include "gofaiWidget.h"
#include "gofaiNewNodeDialog.h"
#include "gofaiNodeEditor.h"
#include "gofaiLinkEditor.h"
#include "gofaiFindDialog.h"
#include "gofaiHistoryDialog.h"
#include "gofaiConfig.h"
#include "gofaiDataQuery.h"
#include "gofaiItemProcess.h"


gofaiWidget::gofaiWidget(QWidget *parent, gofaiDataView *_dv)
    :QWidget(parent, NULL),
     currentNode(_dv),
     currentLink(_dv),
     dv(_dv)
{
    //this->dv = dv;
    // if (rootxpm) delete rootxpm;
    rootxpm=new KRootPixmap(this);

    if (gofaiConfig::getInstance()->transparentBg) {
      rootxpm->setFadeEffect(0.2, QColor(0,0,50));
      rootxpm->start();
    } else {
      rootxpm->stop();
    }
    activezone.setCoords(50,50, 250, 250);
    contextzone.setCoords(0,0,10,10);
    animate_timer = new QTimer(this);
    connect( animate_timer, SIGNAL(timeout()),
             this, SLOT(slotRepaint()) );
    rwtimer = new QTimer(this);
    connect( rwtimer, SIGNAL(timeout()),
             this, SLOT(slotRandomWalk2()) );

    //currentNode.invalidate();
    //currentLink.invalidate();

    setMouseTracking(true);
    setAcceptDrops(TRUE);
    setUpRMM();
}


/**
   Set up the right mouse menus
 */
void gofaiWidget::setUpRMM() {
    /* when clicking on an empty space */
    rmm_empty = new QPopupMenu;
    rmm_empty->insertItem (i18n("New node"),
                                  this,   SLOT(slotNewNode() ));
    rmm_empty->insertItem (i18n("Find node"),
                                  this,   SLOT(slotFind()  ));
    /* when clicking on a node */
    rmm_node = new QPopupMenu;
    rmm_node->insertItem (i18n("Browse to..."),
                                  this,   SLOT(slotBrowseTo() ));
    rmm_node->insertItem (i18n("Edit node"),
                                  this,   SLOT(slotEditNode() ));
    rmm_node->insertItem (BarIcon("editdelete"),
                          i18n("Delete node"),
                          this,   SLOT(slotDeleteNode() ));
    /* when clicking on a link */
    rmm_link = new QPopupMenu;
    rmm_link->insertItem (i18n("Edit link"),
                                  this,   SLOT(slotEditLink() ));
    rmm_link->insertItem (BarIcon("editdelete"),
                          i18n("Delete link"),
                          this,   SLOT(slotDeleteLink() ));
}

void gofaiWidget::paintEvent( QPaintEvent *pe )
{
    QPainter paint( &bufferpixmap );
    bufferpixmap.fill(this, 0, 0);
    drawIt( &paint );
    bitBlt(this, pe->rect().topLeft(), &bufferpixmap, pe->rect());
}

void gofaiWidget::drawIt( QPainter *p )
{
    // paint the context zone:
    KPixmap pm;
    pm.resize(contextzone.width(), contextzone.height());

    QFont font("helvetica", 40);
    p->setFont(font);
    QPen pen;
    pen.setColor(QColor(0,0,0));
    pen.setWidth(3);
    p->setPen(pen);
    p->drawLine(0, contextzone.top()+5, contextzone.right(), contextzone.top()+5);
    p->drawText(contextzone.left()+15,
                contextzone.top()+55,
                "Context zone");
    pen.setColor(QColor(255,255,255));
    p->setPen(pen);
    p->drawLine(0, contextzone.top(), contextzone.right(), contextzone.top());
    p->drawText(contextzone.left()+10,
                contextzone.top()+50,
                "Context zone");

    bool finished = dv->drawIt(p);
    if (!finished) {
      animate_timer->start(10, TRUE); // was 25
    }
}

void gofaiWidget::refresh() {
    dv->getView();
    dv->createVisuals(activezone.left()+activezone.width()/2,
                     activezone.top()+activezone.height()/2,
                     activezone.height()/2);
    repaint(false);
}


void gofaiWidget::mouseReleaseEvent ( QMouseEvent *me) {
    // set new center
    if (me->button() == LeftButton) {
        if ((currentNode.valid) &&
            (currentNode.place == gofaiDataViewPointer::Main )) {
          // get the modified values
            emit saveData(dv->center->val);
            dv->setCenter(currentNode.getVisualItem());
            emit loadData(dv->center->val);
          refresh();
        }
    }

}
void gofaiWidget::mousePressEvent (QMouseEvent *me) {
    static QPoint tmp_point; // not clear if this should be static?
    currentLink.invalidate();
    currentNode.invalidate();
    mousePressPoint = me->pos();
    dv->getNodeFromHere(me->pos(), currentNode);
    dv->getLinkFromHere(me->pos(), currentLink);
    /*
    if (currentNode.valid &&
        currentNode.place==gofaiDataViewPointer::Main) {
      if (currentNode.getItem()->type == Link) {
        currentLink = currentNode;
        currentNode.invalidate();
      }
    }
    */
    if (me->button() == RightButton) {
      tmp_point = QCursor::pos();
      if (currentNode.valid) {
        rmm_node->popup(tmp_point);
      } else {
        if (currentLink.valid) {
          rmm_link->popup(tmp_point);
        } else {
          rmm_empty->popup(tmp_point);
        }
      }
    }
}


void gofaiWidget::mouseDoubleClickEvent ( QMouseEvent *me ) {
  /*
  dv->getItemFromHere(me->pos(), currentNode);
  if (!currentNode.valid) {
    // create a new node
    slotNewNode();
    return;
  }
  // we double clicked on something
  if (currentNode.getItem()->type == Node) {
    slotEditNode();
  } else {
    currentLink = currentNode;
    slotEditLink();
  }
  */
  dv->getNodeFromHere(me->pos(), currentNode);
  if (currentNode.valid) {
    slotEditNode();
    return;
  }
  dv->getLinkFromHere(me->pos(), currentLink);
  if (currentLink.valid) {
    slotEditLink();
    return;
  }
  // just double click on empty space
  slotNewNode();
}


void gofaiWidget::mouseMoveEvent ( QMouseEvent *me ) {
    if (me->state() == LeftButton) {
        // start the drag only after some distance.
      int manhlen = (mousePressPoint-me->pos()).manhattanLength();
      if (manhlen < gofaiConfig::getInstance()->dragInitDistance) {
//        kdDebug() << "Not enough for draginint!" << endl;
        return;
      }
      if (currentNode.valid) {
        // FIXME!!!
        char buf[40];
        sprintf(buf, "GOFAI: %d", currentNode.index);
        QTextDrag *d = new QTextDrag(buf, this );
        d->setSubtype("gofai");
        d->dragCopy();
      }
      kdDebug() << "Current item not valid" << endl;
      return;
    }
    gofaiDataViewPointer oldItem = currentNode;
    dv->getNodeFromHere(me->pos(), currentNode);
    if (!currentNode.valid ||
        currentNode.equals(oldItem) ||
        currentNode.place != gofaiDataViewPointer::Main ) {
        return;
    }
    dv->setActive(currentNode.getItem());
    repaint(false);
}

void gofaiWidget::resizeEvent ( QResizeEvent */*re*/ ) {
    QRect t=rect();
    activezone.setRect(t.width()/4, t.height()/4,
                       t.width()/2,  t.height()/2);
    contextzone.setRect(0, 5 * t.height()/6,
                        t.width(), t.height()/6 );
    bufferpixmap.resize(size());
    // reset the size of the fonts
    gofaiConfig::getInstance()->setScale(t.width(), t.height());
    refresh();
}

void gofaiWidget::moveEvent (QMoveEvent *) {
  //    refresh(); // needed for the transparent bg
}

void gofaiWidget::dragEnterEvent(QDragEnterEvent* event) {
    //event->accept( QTextDrag::canDecode(event) );
    // for the time being accept everything
    event->accept(true);
}

void gofaiWidget::dropEvent(QDropEvent *event) {
  // FIXME: together with the rest
  kdDebug() << "Drop Event" << endl;
  QString text;
  // first check if we dragged this thing over another field
  if (QTextDrag::canDecode(event)) {
        QString a;
        QTextDrag::decode(event,a);
        kdDebug() << "Dragged:" << a << "\n";
        if (a.left(6) == "GOFAI:") {
          kdError() << "Dragging a node\n";
        } else {
          kdError() << "Drag from outside!\n";
          newNodeFromTextDrag(a);
          return;
        }

  }

  gofaiDataViewPointer second(dv);
  dv->getNodeFromHere(event->pos(), second);
  if (second.equals(currentNode)) {
    return;
  }
  // dragged over another node
  if (second.valid) {
    gofaiItem *other;
    other = second.getItem();
    gofaiItem *link = new gofaiItem("reminds-me-of", Link);
    link->setLinkBetween(
                         currentNode.getItem(),
                         other
                         );
    dv->addItem(link);
    refresh();
    return;
    }
  // we dragged this to a node to the context field
  if ((currentNode.place == gofaiDataViewPointer::Main) &&
      contextzone.contains(event->pos())) {
    if ( QTextDrag::decode(event, text) ) {
      kdDebug() << "Ok. Dragged to context" << endl;
      gofaiVisualItem *vis = currentNode.getVisualItem();
      vis->moveTo(event->pos());
      dv->addToContext(vis);
      //kdDebug() << "Refresh called" << endl;
      refresh();
      //kdDebug() << "Now, return" << endl;
      return;
        }
  }
  // dragged it out from the context zone
  if (currentNode.place == gofaiDataViewPointer::Context &&
      !contextzone.contains(event->pos())) {
    kdDebug() << "We dragged it out from the context field!!!" << endl;
    dv->removeFromContext(currentNode);
    refresh();
    return;
  }
  // just rearranging things in the main view

  if (currentNode.place == gofaiDataViewPointer::Main) {
    currentNode.getVisualItem()->moveTo(event->pos());
    repaint(false);
    return;
  }
  // rearranging things in the context zone
  if (currentNode.place == gofaiDataViewPointer::Context &&
      contextzone.contains(event->pos())) {
    kdDebug() << "Rearranging in the context!" << endl;
    //      currentNode.getVisualItem()->moveTo(event->pos());
    gofaiVisualItem *vit = currentNode.getVisualItem();
    vit->moveTo(event->pos());
    vit->setNewPosition();
    repaint(false);
    return;
  }
}

/**
   SLOTS
 */

void gofaiWidget::slotNewNode() {
    gofaiNewNodeDialog nn(this, dv->center->val->name);
    int code = nn.exec();
    if (code == QDialog::Accepted) {
        QString s = nn.getValue();
        gofaiItem *it = new gofaiItem(s, Node);
        dv->addItem(it);
        gofaiConfig::getInstance()->remindsMe = nn.reminds->isChecked();
        if (nn.reminds->isChecked()) {
            gofaiItem *link = new gofaiItem("reminds-me-of", Link);
            link->setLinkBetween(dv->center->val, it);
            dv->addItem(link);
        }
        refresh();
    }
}

void gofaiWidget::newNodeFromTextDrag(QString text) {
    gofaiNewNodeDialog nn(this, dv->center->val->name);
    // only the first word.
    QString title, link;
    int firstblank = text.find(' ');
    if (firstblank != -1) {
        title = text.left(text.find(' '));
    } else {
        title = text;
    }
    if (title.length() > 20) {
        title = title.left(20)+"...";
    }
    // this has to be made smarter
    int firsthttp = text.find("http:",0, false);
    if (firsthttp != -1) {
        int endofhttp = text.find(' ',firsthttp);
        if (endofhttp == -1) {
            link = text.mid(firsthttp, 1000);
        } else {
            link = text.mid(firsthttp, endofhttp - firsthttp);
        }

    }
    nn.edt->setText(title);
    int code = nn.exec();
    if (code == QDialog::Accepted) {
        QString s = nn.getValue();
        gofaiItem *it = new gofaiItem(s, Node);
        it->description = text;
        it->link = link;
        dv->addItem(it);
        gofaiConfig::getInstance()->remindsMe = nn.reminds->isChecked();
        if (nn.reminds->isChecked()) {
            gofaiItem *link = new gofaiItem("reminds-me-of", Link);
            link->setLinkBetween(dv->center->val, it);
            dv->addItem(link);
        }
        refresh();
    }
}

void gofaiWidget::slotDeleteNode() {
    if (gofaiConfig::getInstance()->askDelete) {
      int value = KMessageBox::warningYesNo( this,"Really delete this node?");
      if (value != KMessageBox::Yes) {
        return;
      }
    }
    dv->removeItem(currentNode.getItem());
    refresh();
}

/**
   Edits a node
 */
void gofaiWidget::slotEditNode() {
    // edit the current node
    emit saveData(dv->center->val);
    gofaiNodeEditor ne(this);
    gofaiItem *edited = currentNode.getItem();
    ne.setFields(edited);
    int code = ne.exec();
    if (code == QDialog::Accepted) {
        if (ne.changed) {
            ne.getFields(edited);
            dv->updateItem(edited);
            emit loadData(dv->center->val);
            repaint(false);
        }
    }
}

/**
   Edits a node
 */
void gofaiWidget::slotBrowseTo() {
    gofaiItemProcess::browseTo(currentNode.getItem());
}


/**
   Edits a link
 */
void gofaiWidget::slotEditLink() {
    gofaiLinkEditor le(this);
    gofaiItem *edited = currentLink.getItem();
    le.setFields(edited);
    int code = le.exec();
    if (code == QDialog::Accepted) {
        if (le.changed) {
            le.getFields(edited);
            dv->updateItem(edited);
            repaint(false);
        }
    }
}

void gofaiWidget::slotDeleteLink() {
    if (gofaiConfig::getInstance()->askDelete) {
      int value = KMessageBox::warningYesNo( this,
                                             "Really delete this link?");
      if (value != KMessageBox::Yes) {
        return;
      }
    }
    dv->removeItem(currentLink.getItem());
    refresh();
}

void gofaiWidget::slotFind() {
   gofaiFindDialog fd(this, dv);
   int fd_res = fd.exec();
   if (fd_res != QDialog::Accepted) {
       return;
   }
   QString val = fd.getValue();
   gofaiItem *it = dv->getItemByName(val);
   if (it != NULL) {
       emit loadData(dv->center->val);
       dv->center->val = it; // a bit of a hack
       dv->setCenter(dv->center);
       refresh();
   } else {
       QString mes;
       mes.sprintf("Node %s not found!", val.ascii());
       KMessageBox::sorry(this,"Node not found!");
   }
}

void gofaiWidget::slotHistory() {
    gofaiHistoryDialog hd(this);
    // fill in the list box
    QListIterator<gofaiItem> iter(dv->history);
    for ( ; iter.current(); ++iter ) {
        gofaiItem *it = iter.current();
        hd.lb->insertItem(it->name);
    }
    hd.lb->setSelected(0, TRUE);
    int hd_res = hd.exec();
    if (hd_res != QDialog::Accepted) {
        return;
    }
   QString val = hd.getValue();
   gofaiItem *it = dv->getItemByName(val);
   if (it != NULL) {
       dv->center->val = it; // a bit of a hack
       dv->setCenter(dv->center);
       refresh();
   } else {
       QString mes;
       mes.sprintf("Node %s not found - maybe it was deleted!", val.ascii());
       KMessageBox::sorry(this, "Node not found!");
   }

}

void gofaiWidget::slotRepaint() {
    repaint(false);
}

void gofaiWidget::slotRandomWalk() {
    rwcount = 5;
    slotRandomWalk2();
}

void gofaiWidget::slotRandomWalk2() {
    // set a new center, from those connected
  // FIXME
  /*
    for(int i = 0; i!=dv->count; i++) {
        // later, go random
        if (dv->vds[i]!=dv->center && dv->vds[i]->val->type == Node)
            dv->setCenter(dv->vds[i]);
    }
    refresh();
    rwcount--;
    if (rwcount > 0)
        rwtimer->start(1500, TRUE);
  */
}

#include "gofaiWidget.moc"
